@@ -1,5 +1,5 @@
 /*  smplayer, GUI front-end for mplayer.
-    Copyright (C) 2006-2013 Ricardo Villalba <rvm@users.sourceforge.net>
+    Copyright (C) 2006-2023 Ricardo Villalba <ricardo@smplayer.info>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,8 +16,8 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-#ifndef _COLORUTILS_H_
-#define _COLORUTILS_H_
+#ifndef COLORUTILS_H
+#define COLORUTILS_H
 
 #include <QString>
 
@@ -25,27 +25,36 @@
 #define COLOR_OUTPUT_SUPPORT 1
 #endif
 
+//#define CHANGE_WIDGET_COLOR
+
 class QWidget;
 class QColor;
 
 class ColorUtils {
 
 public:
-
-	//! Returns a string suitable to be used for -ass-color
+#if 0
+	static QString colorToRGB(unsigned int color);
 	static QString colorToRRGGBBAA(unsigned int color);
 	static QString colorToRRGGBB(unsigned int color);
-
-	//! Returns a string suitable to be used for -colorkey
-	static QString colorToRGB(unsigned int color);
-
 	static QString colorToAABBGGRR(unsigned int color);
+#endif
 
+	static QString colorToRGB(QColor c);
+	static QString colorToRRGGBBAA(QColor c);
+	static QString colorToRRGGBB(QColor c);
+	static QString colorToAABBGGRR(QColor c);
+	static QString colorToAARRGGBB(QColor c);
+
+	static QColor AARRGGBBToColor(const QString & s);
+
+#ifdef CHANGE_WIDGET_COLOR
 	//! Changes the foreground color of the specified widget
 	static void setForegroundColor(QWidget * w, const QColor & color);
 
 	//! Changes the background color of the specified widget
 	static void setBackgroundColor(QWidget * w, const QColor & color);
+#endif
 
     /**
      ** \brief Strip colors and tags from MPlayer output lines
