@@ -1,5 +1,5 @@
 /*  smplayer, GUI front-end for mplayer.
-    Copyright (C) 2006-2017 Ricardo Villalba <rvm@users.sourceforge.net>
+    Copyright (C) 2006-2023 Ricardo Villalba <ricardo@smplayer.info>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -25,8 +25,10 @@
 #include "paths.h"
 #include "vdpauproperties.h"
 #include "playerid.h"
+#include "helper.h"
+#include <QDebug>
 
-#if USE_ALSA_DEVICES || USE_PULSEAUDIO_DEVICES || USE_XV_ADAPTORS || USE_DSOUND_DEVICES
+#if USE_ALSA_DEVICES || USE_PULSEAUDIO_DEVICES || USE_XV_ADAPTORS || USE_DSOUND_DEVICES || USE_MPV_PULSE_DEVICES
 #include "deviceinfo.h"
 #endif
 
@@ -58,19 +60,16 @@ PrefGeneral::PrefGeneral(QWidget * parent, Qt::WindowFlags f)
 	vo_list = i->voList();
 	ao_list = i->aoList();
 
-#if USE_DSOUND_DEVICES
+#ifdef AUDIO_DEVICES_SELECTION
+	#if USE_DSOUND_DEVICES
 	dsound_devices = DeviceInfo::dsoundDevices();
+	#endif
+
+	#if USE_ALSA_DEVICES
+	alsa_devices = DeviceInfo::alsaDevices();
+	#endif
 #endif
 
-#if USE_ALSA_DEVICES
-	alsa_devices = DeviceInfo::alsaDevices();
-#endif
-#if USE_MPV_ALSA_DEVICES
-	mpv_alsa_devices = DeviceInfo::mpvAlsaDevices();
-#endif
-#if USE_PULSEAUDIO_DEVICES
-	pa_devices = DeviceInfo::paDevices();
-#endif
 #if USE_XV_ADAPTORS
 	xv_adaptors = DeviceInfo::xvAdaptors();
 #endif
@@ -85,10 +84,14 @@ PrefGeneral::PrefGeneral(QWidget * parent, Qt::WindowFlags f)
 	avoid_screensaver_check->hide();
 	#endif
 #else
+	#ifndef SCREENSAVER_OFF
+	screensaver_check->hide();
+	#endif
 	screensaver_group->hide();
 #endif
 
-#if defined(Q_OS_WIN) || defined(Q_OS_OS2)
+#ifndef OS_UNIX_NOT_MAC
+	wayland_check->hide();
 	vdpau_button->hide();
 #endif
 
@@ -104,8 +107,13 @@ PrefGeneral::PrefGeneral(QWidget * parent, Qt::WindowFlags f)
 	blackborders_check->hide();
 #endif
 
+#if !SELECT_TRACKS_ON_STARTUP
+	tracks_frame->hide();
+	tracks_separator->hide();
+#endif
+
 #ifdef MPV_SUPPORT
-	screenshot_format_combo->addItems(QStringList() << "png" << "ppm" << "pgm" << "pgmyuv" << "tga" << "jpg" << "jpeg");
+	screenshot_format_combo->addItems(QStringList() << "png" << "jpg" << "webp");
 #else
 	screenshot_template_label->hide();
 	screenshot_template_edit->hide();
@@ -184,7 +192,9 @@ void PrefGeneral::retranslateStrings() {
 	filesettings_method_combo->addItem( tr("multiple ini files"), "hash");
 	filesettings_method_combo->setCurrentIndex(filesettings_method_item);
 
+#ifndef AUDIO_DEVICES_SELECTION
 	updateDriverCombos();
+#endif
 
     // Icons
 	/*
@@ -206,9 +216,7 @@ void PrefGeneral::retranslateStrings() {
            "subtitle streams is found, SMPlayer will try to use your "
            "preferred language. This only will work with media that offer "
            "info about the language of audio and subtitle streams, like DVDs "
-           "or mkv files.<br>These fields accept regular expressions. "
-           "Example: <b>es|esp|spa</b> will select the track if it matches with "
-            "<i>es</i>, <i>esp</i> or <i>spa</i>."));
+           "or mkv files.") );
 
 #ifndef MPLAYER_MPV_SELECTION
 	executable_label->setText( tr("%1 &executable:").arg(PLAYER_NAME) );
@@ -235,10 +243,19 @@ void PrefGeneral::setData(Preferences * pref) {
 	remember_streams_check->setChecked(pref->remember_stream_settings);
 
 	setFileSettingsMethod( pref->file_settings_method );
+
+#if SIMPLE_TRACK_SELECTION
+	setAudioLang( pref->alang );
+	setSubtitleLang( pref->slang );
+#else
 	setAudioLang( pref->audio_lang );
 	setSubtitleLang( pref->subtitle_lang );
+#endif
+#if SELECT_TRACKS_ON_STARTUP
 	setAudioTrack( pref->initial_audio_track );
 	setSubtitleTrack( pref->initial_subtitle_track );
+#endif
+
 	setCloseOnFinish( pref->close_on_finish );
 	setPauseWhenHidden( pref->pause_when_hidden );
 #ifdef AUTO_SHUTDOWN_PC
@@ -273,16 +290,21 @@ void PrefGeneral::setData(Preferences * pref) {
 
 #ifdef Q_OS_WIN
 	#ifdef SCREENSAVER_OFF
-	setTurnScreensaverOff( pref->turn_screensaver_off );
+	setTurnScreensaverOff( pref->disable_screensaver );
 	#endif
 	#ifdef AVOID_SCREENSAVER
 	setAvoidScreensaver( pref->avoid_screensaver );
 	#endif
 #else
+	#ifdef SCREENSAVER_OFF
 	setDisableScreensaver( pref->disable_screensaver );
+	#endif
+	#ifdef OS_UNIX_NOT_MAC
+	wayland_check->setChecked(pref->wayland_workarounds);
+	#endif
 #endif
 
-#if !defined(Q_OS_WIN) && !defined(Q_OS_OS2)
+#ifdef OS_UNIX_NOT_MAC
 	vdpau = pref->vdpau;
 #endif
 
@@ -294,15 +316,54 @@ void PrefGeneral::setData(Preferences * pref) {
 
 	setMcActivated( pref->use_mc );
 	setMc( pref->mc_value );
+
+#ifdef AUDIO_DEVICES_SELECTION
+#if USE_PULSEAUDIO_DEVICES
+	#if USE_MPV_PULSE_DEVICES
+	if (PlayerID::player(pref->mplayer_bin) == PlayerID::MPLAYER)
+	#endif
+	{
+		pa_devices = DeviceInfo::paDevices();
+	}
+#endif
+#if MPV_AUDIO_DEVICES
+	if (PlayerID::player(pref->mplayer_bin) == PlayerID::MPV) {
+		#if USE_MPV_PULSE_DEVICES
+		pa_devices = DeviceInfo::mpvPulseDevices();
+		#endif
+		#if USE_MPV_ALSA_DEVICES
+		mpv_alsa_devices = DeviceInfo::mpvAlsaDevices();
+		#endif
+		#if USE_MPV_COREAUDIO_DEVICES
+		coreaudio_devices = DeviceInfo::mpvCoreaudioDevices();
+		#endif
+	}
+#endif
+	updateDriverCombos();
+#endif // AUDIO_DEVICES_SELECTION
+
+	global_speed_check->setChecked(pref->global_speed);
 }
 
 void PrefGeneral::getData(Preferences * pref) {
 	requires_restart = false;
 	filesettings_method_changed = false;
 
-	if (pref->mplayer_bin != mplayerPath()) {
+	QString player_path = mplayerPath();
+#ifdef MPLAYER_MPV_SELECTION
+	#ifdef OS_UNIX_NOT_MAC
+	if (player_path == PLAYER_COMBO_MPLAYER_PATH || player_path == PLAYER_COMBO_MPV_PATH || !QFile::exists(player_path)) {
+		QString player_name = QFileInfo(player_path).fileName();
+		QString found_player = Helper::findExecutable(player_name);
+		if (!found_player.isEmpty()) player_path = found_player;
+	}
+	#endif
+#endif
+	qDebug() << "PrefGeneral::getData: player_path:" << player_path;
+
+	if (pref->mplayer_bin != player_path) {
 		requires_restart = true;
-		pref->mplayer_bin = mplayerPath();
+		pref->mplayer_bin = player_path;
 
 		qDebug("PrefGeneral::getData: mplayer binary has changed, getting version number");
 		// Forces to get info from mplayer to update version number
@@ -333,11 +394,17 @@ void PrefGeneral::getData(Preferences * pref) {
 		filesettings_method_changed = true;
 	}
 
+#if SIMPLE_TRACK_SELECTION
+	pref->alang = audioLang();
+	pref->slang = subtitleLang();
+#else
 	pref->audio_lang = audioLang();
 	pref->subtitle_lang = subtitleLang();
-
+#endif
+#if SELECT_TRACKS_ON_STARTUP
 	pref->initial_audio_track = audioTrack();
 	pref->initial_subtitle_track = subtitleTrack();
+#endif
 
 	pref->close_on_finish = closeOnFinish();
 	pref->pause_when_hidden = pauseWhenHidden();
@@ -377,16 +444,24 @@ void PrefGeneral::getData(Preferences * pref) {
 
 #ifdef Q_OS_WIN
 	#ifdef SCREENSAVER_OFF
-	TEST_AND_SET(pref->turn_screensaver_off, turnScreensaverOff());
+	TEST_AND_SET(pref->disable_screensaver, turnScreensaverOff());
 	#endif
 	#ifdef AVOID_SCREENSAVER
 	pref->avoid_screensaver = avoidScreensaver();
 	#endif
 #else
+	#ifdef SCREENSAVER_OFF
 	TEST_AND_SET(pref->disable_screensaver, disableScreensaver());
+	#endif
+	#ifdef OS_UNIX_NOT_MAC
+	if (pref->wayland_workarounds != wayland_check->isChecked()) {
+		requires_restart = true;
+		pref->wayland_workarounds = wayland_check->isChecked();
+	}
+	#endif
 #endif
 
-#if !defined(Q_OS_WIN) && !defined(Q_OS_OS2)
+#ifdef OS_UNIX_NOT_MAC
 	pref->vdpau = vdpau;
 #endif
 
@@ -398,6 +473,8 @@ void PrefGeneral::getData(Preferences * pref) {
 
 	TEST_AND_SET(pref->use_mc, mcActivated());
 	TEST_AND_SET(pref->mc_value, mc());
+
+	pref->global_speed = global_speed_check->isChecked();
 }
 
 void PrefGeneral::updateDriverCombos() {
@@ -415,11 +492,11 @@ void PrefGeneral::updateDriverCombos() {
 		vo = vo_list[n].name();
 		#ifdef Q_OS_WIN
 		if ( vo == "directx" ) {
-			vo_combo->addItem( "directx (" + tr("fast") + ")", "directx" );
-			vo_combo->addItem( "directx (" + tr("slow") + ")", "directx:noaccel" );
+			vo_combo->addItem( "directx (" + tr("hardware") + ")", "directx" );
+			vo_combo->addItem( "directx (" + tr("software") + ")", "directx:noaccel" );
 		}
 		else
-		#else
+		#endif
 		#ifdef Q_OS_OS2
 		if ( vo == "kva") {
 			vo_combo->addItem( "kva (" + tr("fast") + ")", "kva" );
@@ -427,11 +504,7 @@ void PrefGeneral::updateDriverCombos() {
 			vo_combo->addItem( "kva (" + tr("slower dive mode") + ")", "kva:dive" );
 		}
 		else
-		#else
-		/*
-		if (vo == "xv") vo_combo->addItem( "xv (" + tr("fastest") + ")", vo);
-		else
-		*/
+		#endif
 		#if USE_XV_ADAPTORS
 		if ((vo == "xv") && (!xv_adaptors.isEmpty())) {
 			vo_combo->addItem(vo, vo);
@@ -441,30 +514,21 @@ void PrefGeneral::updateDriverCombos() {
 			}
 		}
 		else
-		#endif // USE_XV_ADAPTORS
 		#endif
+		#ifdef USE_SHM
+		if (vo == "shm") {
+			vo_combo->addItem( "Shared memory", vo);
+		}
+		else
 		#endif
-		if (vo == "x11") vo_combo->addItem( "x11 (" + tr("slow") + ")", vo);
-		else
-		if (vo == "gl") {
-			vo_combo->addItem( vo, vo);
-			vo_combo->addItem( "gl (" + tr("fast") + ")", "gl:yuv=2:force-pbo");
-			vo_combo->addItem( "gl (" + tr("fast - ATI cards") + ")", "gl:yuv=2:force-pbo:ati-hack");
-			vo_combo->addItem( "gl (yuv)", "gl:yuv=3");
+		#ifdef Q_OS_MACX
+		if (vo == "sharedbuffer") {
+			vo_combo->addItem( "Cocoa shared buffer", vo);
 		}
 		else
-		if (vo == "gl2") {
-			vo_combo->addItem( vo, vo);
-			vo_combo->addItem( "gl2 (yuv)", "gl2:yuv=3");
-		}
-		else
-		if (vo == "gl_tiled") {
-			vo_combo->addItem( vo, vo);
-			vo_combo->addItem( "gl_tiled (yuv)", "gl_tiled:yuv=3");
-		}
-		else
-		if (vo == "null" || vo == "png" || vo == "jpeg" || vo == "gif89a" || 
-            vo == "tga" || vo == "pnm" || vo == "md5sum" ) 
+		#endif
+		if (/*vo == "libmpv" ||*/ vo == "null" || vo == "png" || vo == "jpeg" || vo == "gif89a" ||
+            vo == "tga" || vo == "pnm" || vo == "md5sum" || vo == "image" || vo == "tct" || vo == "yuv4mpeg")
 		{
 			; // Nothing to do
 		}
@@ -485,6 +549,7 @@ void PrefGeneral::updateDriverCombos() {
 		}
 		#endif
 		*/
+		#ifdef AUDIO_DEVICES_SELECTION
 		#if USE_ALSA_DEVICES
 		if ((ao == "alsa") && (!alsa_devices.isEmpty())) {
 			for (int n=0; n < alsa_devices.count(); n++) {
@@ -499,13 +564,20 @@ void PrefGeneral::updateDriverCombos() {
 			}
 		}
 		#endif
-		#if USE_PULSEAUDIO_DEVICES
+		#if USE_PULSEAUDIO_DEVICES || USE_MPV_PULSE_DEVICES
 		if ((ao == "pulse") && (!pa_devices.isEmpty())) {
 			for (int n=0; n < pa_devices.count(); n++) {
 				ao_combo->addItem( DeviceInfo::printableName("pulse", pa_devices[n]), DeviceInfo::internalName("pulse", pa_devices[n]) );
 			}
 		}
 		#endif
+		#if USE_MPV_COREAUDIO_DEVICES
+		if ((ao == "coreaudio") && (!coreaudio_devices.isEmpty())) {
+			for (int n=0; n < coreaudio_devices.count(); n++) {
+				ao_combo->addItem( DeviceInfo::printableName("coreaudio", coreaudio_devices[n]), DeviceInfo::internalName("coreaudio", coreaudio_devices[n]) );
+			}
+		}
+		#endif
 		#if USE_DSOUND_DEVICES
 		if ((ao == "dsound") && (!dsound_devices.isEmpty())) {
 			for (int n=0; n < dsound_devices.count(); n++) {
@@ -513,6 +585,7 @@ void PrefGeneral::updateDriverCombos() {
 			}
 		}
 		#endif
+		#endif // AUDIO_DEVICES_SELECTION
 	}
 	ao_combo->addItem( tr("User defined..."), "user_defined" );
 
@@ -672,6 +745,7 @@ QString PrefGeneral::subtitleLang() {
 	return subtitle_lang_edit->text();
 }
 
+#if SELECT_TRACKS_ON_STARTUP
 void PrefGeneral::setAudioTrack(int track) {
 	audio_track_spin->setValue(track);
 }
@@ -687,6 +761,7 @@ void PrefGeneral::setSubtitleTrack(int track) {
 int PrefGeneral::subtitleTrack() {
 	return subtitle_track_spin->value();
 }
+#endif
 
 void PrefGeneral::setCloseOnFinish(bool b) {
 	close_on_finish_check->setChecked(b);
@@ -946,6 +1021,10 @@ void PrefGeneral::player_combo_changed(int idx) {
 		player_spacer->changeSize(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);
 		mplayerbin_edit->setVisible(false);
 	}
+
+	#ifdef OS_UNIX_NOT_MAC
+	wayland_check->setVisible(mplayerPath().contains("mpv"));
+	#endif
 }
 #endif
 
@@ -955,7 +1034,7 @@ void PrefGeneral::vo_combo_changed(int idx) {
 	vo_user_defined_edit->setVisible(visible);
 	vo_user_defined_edit->setFocus();
 
-#ifndef Q_OS_WIN
+#ifdef OS_UNIX_NOT_MAC
 	bool vdpau_button_visible = (vo_combo->itemData(idx).toString() == "vdpau" && mplayerPath().contains("mplayer"));
 	vdpau_button->setVisible(vdpau_button_visible);
 #endif
@@ -968,7 +1047,7 @@ void PrefGeneral::ao_combo_changed(int idx) {
 	ao_user_defined_edit->setFocus();
 }
 
-#ifndef Q_OS_WIN
+#ifdef OS_UNIX_NOT_MAC
 void PrefGeneral::on_vdpau_button_clicked() {
 	qDebug("PrefGeneral::on_vdpau_button_clicked");
 
@@ -1066,6 +1145,10 @@ void PrefGeneral::createHelp() {
 		);
 #endif
 
+	setWhatsThis(global_speed_check, tr("Keep selected speed across files"),
+		tr("If this option is enabled, the speed selected in the Play menu will be applied for all files.") +" "+
+		tr("Otherwise each file will use its own speed setting."));
+
 	setWhatsThis(pause_if_hidden_check, tr("Pause when minimized"),
 		tr("If this option is enabled, the file will be paused when the "
            "main window is hidden. When the window is restored, playback "
@@ -1086,12 +1169,15 @@ void PrefGeneral::createHelp() {
 	setWhatsThis(vo_combo, tr("Video output driver"),
 		tr("Select the video output driver."));
 
-#if !defined(Q_OS_WIN) && !defined(Q_OS_OS2)
+#ifdef OS_UNIX_NOT_MAC
 	/*
 	setWhatsThis(vdpau_filters_check, tr("Disable video filters when using vdpau"),
 		tr("Usually video filters won't work when using vdpau as video output "
            "driver, so it's wise to keep this option checked.") );
 	*/
+
+	setWhatsThis(wayland_check, tr("Wayland support"),
+		tr("This activates some options to prevent the video being displayed outside the main window."));
 #endif
 
 	setWhatsThis(postprocessing_check, tr("Enable postprocessing by default"),
@@ -1190,7 +1276,7 @@ void PrefGeneral::createHelp() {
 		tr("If it's not checked, the audio equalizer values are saved along each file "
            "and loaded back when the file is played later.") );
 
-	setWhatsThis(hwac3_check, tr("AC3/DTS pass-through S/PDIF"),
+	setWhatsThis(hwac3_check, tr("AC3/DTS passthrough over S/PDIF and HDMI"),
 		tr("Uses hardware AC3 passthrough.") + "<br>" +
         tr("<b>Note:</b> none of the audio filters will be used when this "
            "option is enabled.") );
@@ -1239,25 +1325,29 @@ void PrefGeneral::createHelp() {
 	addSectionTitle(tr("Preferred audio and subtitles"));
 
 	setWhatsThis(audio_lang_edit, tr("Preferred audio language"),
-		tr("Here you can type your preferred language for the audio streams. "
-           "When a media with multiple audio streams is found, SMPlayer will "
-           "try to use your preferred language.<br>"
-           "This only will work with media that offer info about the language "
-           "of the audio streams, like DVDs or mkv files.<br>"
-           "This field accepts regular expressions. Example: <b>es|esp|spa</b> "
+#if SIMPLE_TRACK_SELECTION
+		tr("You can specify here a priority list of audio language codes, separated by commas. "
+           "For example: spa,eng,jpn")
+
+#else
+		tr("This field accepts regular expressions. Example: <b>es|esp|spa</b> "
            "will select the audio track if it matches with <i>es</i>, "
-           "<i>esp</i> or <i>spa</i>.") );
+           "<i>esp</i> or <i>spa</i>.")
+#endif
+	);
 
 	setWhatsThis(subtitle_lang_edit, tr("Preferred subtitle language"),
-		tr("Here you can type your preferred language for the subtitle stream. "
-           "When a media with multiple subtitle streams is found, SMPlayer will "
-           "try to use your preferred language.<br>"
-           "This only will work with media that offer info about the language "
-           "of the subtitle streams, like DVDs or mkv files.<br>"
-           "This field accepts regular expressions. Example: <b>es|esp|spa</b> "
+#if SIMPLE_TRACK_SELECTION
+		tr("You can specify here a priority list of subtitle language codes, separated by commas. "
+           "For example: spa,eng,jpn")
+#else
+		tr("This field accepts regular expressions. Example: <b>es|esp|spa</b> "
            "will select the subtitle stream if it matches with <i>es</i>, "
-           "<i>esp</i> or <i>spa</i>.") );
+           "<i>esp</i> or <i>spa</i>.")
+#endif
+	);
 
+#if SELECT_TRACKS_ON_STARTUP
 	setWhatsThis(audio_track_spin, tr("Audio track"),
 		tr("Specifies the default audio track which will be used when playing "
            "new files. If the track doesn't exist, the first one will be used. "
@@ -1269,7 +1359,7 @@ void PrefGeneral::createHelp() {
            "playing new files. If the track doesn't exist, the first one "
            "will be used. <br><b>Note:</b> the <i>\"preferred subtitle "
            "language\"</i> has preference over this option.") );
-
+#endif
 }
 
 #include "moc_prefgeneral.cpp"
