@@ -1,5 +1,5 @@
 /*  smplayer, GUI front-end for mplayer.
-    Copyright (C) 2006-2013 Ricardo Villalba <rvm@users.sourceforge.net>
+    Copyright (C) 2006-2023 Ricardo Villalba <ricardo@smplayer.info>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -34,7 +34,8 @@ int main( int argc, char ** argv )
 
 	QSettings set(QSettings::IniFormat, QSettings::UserScope, "RVM", "videopreview");
 
-	VideoPreview vp("mplayer");
+	//VideoPreview vp("mplayer");
+	VideoPreview vp("mpv");
 	vp.setSettings(&set);
 
 	if (!filename.isEmpty())
